Attribute VB_Name = "Helper"
' ====================================================================
' Helper module
'
' Declarations for clipboard api
' Clipboard constants
' hex dump and formatting functions
'
' ====================================================================
Option Explicit

' Predefined Clipboard Formats
Public Const CF_TEXT = 1
Public Const CF_BITMAP = 2
Public Const CF_METAFILEPICT = 3
Public Const CF_SYLK = 4
Public Const CF_DIF = 5
Public Const CF_TIFF = 6
Public Const CF_OEMTEXT = 7
Public Const CF_DIB = 8
Public Const CF_PALETTE = 9
Public Const CF_PENDATA = 10
Public Const CF_RIFF = 11
Public Const CF_WAVE = 12
Public Const CF_UNICODETEXT = 13
Public Const CF_ENHMETAFILE = 14
Public Const CF_HDROP = 15
Public Const CF_LOCALE = 16
Public Const CF_MAX = 17
Public Const CF_OWNERDISPLAY = &H80
Public Const CF_DSPTEXT = &H81
Public Const CF_DSPBITMAP = &H82
Public Const CF_DSPMETAFILEPICT = &H83
Public Const CF_DSPENHMETAFILE = &H8E

' clipboard API function
Public Declare Function OpenClipboard Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function CloseClipboard Lib "user32" () As Long
Public Declare Function GetClipboardOwner Lib "user32" () As Long
Public Declare Function SetClipboardViewer Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function GetClipboardViewer Lib "user32" () As Long
Public Declare Function ChangeClipboardChain Lib "user32" (ByVal hWnd As Long, ByVal hWndNext As Long) As Long
Public Declare Function SetClipboardData Lib "user32" (ByVal wFormat As Long, ByVal hMem As Long) As Long
Public Declare Function GetClipboardData Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function RegisterClipboardFormat Lib "user32" Alias "RegisterClipboardFormatA" (ByVal lpString As String) As Long
Public Declare Function CountClipboardFormats Lib "user32" () As Long
Public Declare Function EnumClipboardFormats Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function GetClipboardFormatName Lib "user32" Alias "GetClipboardFormatNameA" (ByVal wFormat As Long, ByVal lpString As String, ByVal nMaxCount As Long) As Long
Public Declare Function EmptyClipboard Lib "user32" () As Long
Public Declare Function IsClipboardFormatAvailable Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function GetPriorityClipboardFormat Lib "user32" (lpPriorityList As Long, ByVal nCount As Long) As Long
Public Declare Function GetOpenClipboardWindow Lib "user32" () As Long

' General Purpose API stuff
Public Declare Function GlobalAlloc Lib "Kernel32" _
  (ByVal wFlags As Long, ByVal dwBytes As Long) As Long
Public Declare Function GlobalSize Lib "Kernel32" _
   (ByVal hMem As Long) As Long
Public Declare Function GlobalLock Lib "Kernel32" _
   (ByVal hMem As Long) As Long
Public Declare Function GlobalUnlock Lib "Kernel32" _
   (ByVal hMem As Long) As Long
Public Declare Sub CopyMemory Lib "Kernel32" Alias "RtlMoveMemory" _
   (Destination As Any, Source As Any, ByVal length As Long)
Public Declare Sub ZeroMemory Lib "Kernel32" Alias "RtlMoveMemory" _
  (dest As Any, ByVal numBytes As Long)

' needed to autoregister the binder dll
Const DLLNAME = "atlbind.dll"
Public Declare Function RegisterComponent Lib "atlbind.dll" Alias "DllRegisterServer" () As Long

Sub Main()
  Dim obind As Object
  Dim ok As Boolean
  Dim res As Long
  
  ' test create instance of the binder component
  On Error Resume Next
  Set obind = New ATLBINDLib.Binder
  ok = (Err.Number = 0)
  On Error GoTo 0
  
  If Not ok Then
    ' not successfull
    ' set curdir to app.path
    ChDir App.Path
    ChDrive App.Path
    ' check if dll exists
    If Dir$(DLLNAME) = "" Then ChDir ".."
    ' check parent path
    If Dir$(DLLNAME) = "" Then
      ' dll does not exist
      MsgBox "Unable to locate the component " & DLLNAME & vbCrLf & _
        "in path " & CurDir$, vbCritical
      End
    End If
    
    ' dll exists - now call DllRegisterServer on that dll
    Err.Clear
    On Error Resume Next
    res = RegisterComponent
    ok = (Err.Number = 0) And (res = 0)
    On Error GoTo 0
    If Not ok Then
      MsgBox "Failure during registration of the component" & vbCrLf & _
        "res=" & CStr(res) & vbCrLf & _
        CStr(Err.Number) & "=" & Err.Description, _
        vbCritical
      End
    End If
  End If
  
  ' dll registered (and fully functional?!)
  frmClipx.Show
End Sub

' Convert a clipboard format identifier to a readable string
Public Function GetClipFormatName(ByVal nFmt As Long) As String
  Dim nRet As Long
  Dim sFmt As String
  Const MAXCHARS = 1024
  
  Select Case nFmt
    Case CF_TEXT
      sFmt = "Text (TXT)"
    Case CF_BITMAP
      sFmt = "Bitmap (BMP)"
    Case CF_METAFILEPICT
      sFmt = "Metafile (WMF)"
    Case CF_SYLK
      sFmt = "Microsoft Symbolic Link (SYLK)"
    Case CF_DIF
      sFmt = "Data Interchange Format (DIF)"
    Case CF_TIFF
      sFmt = "Tagged Interchange File Format (TIF)"
    Case CF_OEMTEXT
      sFmt = "OEM Text"
    Case CF_DIB
      sFmt = "Device Independent Bitmap (DIB)"
    Case CF_PALETTE
      sFmt = "Palette"
    Case CF_PENDATA
      sFmt = "Pen Data"
    Case CF_RIFF
      sFmt = "RIFF"
    Case CF_WAVE
      sFmt = "Wave"
    Case CF_UNICODETEXT
      sFmt = "Unicode Text"
    Case CF_ENHMETAFILE
      sFmt = "Enhanced Metafile (EMF)"
    Case CF_HDROP
      sFmt = "Dropped Filelist"
    Case CF_LOCALE
      sFmt = "Locale Identifier"
    Case Else
      sFmt = String(MAXCHARS, 0)
      nRet = GetClipboardFormatName(nFmt, sFmt, MAXCHARS)
      If nRet = 0 Then
         sFmt = "<Unknown>"
      Else
         sFmt = Left$(sFmt, nRet) & "  (" & CStr(nFmt) & ")"
      End If
  End Select
  GetClipFormatName = sFmt
End Function

' Convert a byte array to a hex/ascii dump
Public Function ByteArrayToHex(ByRef arBuffer() As Byte) As String
  ' get data as hex / ascii
  Dim lbuffersize As Long
  Dim shex As String, sascii As String, ch As String, sLines As String, bc As Long, i As Long
  Dim idx As Long
  Dim arrLines() As String
  Const PERLINE = 16
  Const CHUNK = 10
  Dim lineindex As Long
  
  lbuffersize = UBound(arBuffer) - LBound(arBuffer) + 1
  For i = LBound(arBuffer) To UBound(arBuffer)
    bc = arBuffer(i)
    shex = shex & fhex(bc, 2) & " "
    If bc < 32 Then bc = Asc("_")
    sascii = sascii & Chr$(bc)
    If ((i > 0) And (i Mod PERLINE = PERLINE - 1)) Or (i = UBound(arBuffer)) Then
      shex = Left$(shex & Space$(PERLINE * 3), PERLINE * 3)
      If (i = UBound(arBuffer)) Then
        idx = ((i \ PERLINE)) * PERLINE
      Else
        idx = i - PERLINE + 1
      End If
      addline arrLines, fhex(idx, 8) & ": " & shex & Space(2) & sascii, CHUNK, lineindex
      lineindex = lineindex + 1
      shex = ""
      sascii = ""
    End If
  Next
  ByteArrayToHex = Join(arrLines, vbCrLf)
End Function

' convert a long to a hexadicimal string with a specific size (prepend zeros)
Public Function fhex(lValue As Long, Size As Long) As String
  fhex = Right$(String$(Size, "0") & Hex$(lValue), Size)
End Function

' add a line to a string array
' automatically adjusting size in chunks
Private Sub addline(strArray() As String, line As String, CHUNK As Long, lineindex As Long)
  Dim top As Long, ok As Boolean
  On Error Resume Next
  top = UBound(strArray)
  ok = (Err.Number = 0)
  On Error GoTo 0
  If ok Then
    ' resize array with extra chunk
    If lineindex > top Then
      top = top + CHUNK
      ReDim Preserve strArray(0 To top) As String
    End If
  Else
    ' array never dimensioned before
    ReDim strArray(0 To CHUNK) As String
  End If
  strArray(lineindex) = line
End Sub

' convert part of a byte array to a hexadecimal string
Public Function fhexmul(arBuf As Variant, start As Long, numb As Long) As String
  Dim i As Long
  Dim sres As String
  For i = start To start + numb - 1
    If i > UBound(arBuf) Then Exit For
    sres = sres & fhex((arBuf(i)), 2)
  Next
  fhexmul = sres
End Function

' convert part of a byte array to a string
Public Function fstrmul(arBuf As Variant, start As Long, numb As Long) As String
  Dim i As Long
  Dim sres As String
  For i = start To start + numb - 1
    If i > UBound(arBuf) Then Exit For
    sres = sres & Chr$(arBuf(i))
  Next
  fstrmul = sres
End Function

