// Binder.cpp : Implementation of CBinder
#include "stdafx.h"
#include "AtlBind.h"
#include "Binder.h"

/////////////////////////////////////////////////////////////////////////////
// CBinder


STDMETHODIMP CBinder::Bind(long hwnd)
{
	HRESULT hr;

  /* This call might fail, in which case OLE sets m_pdth = NULL */
  CoCreateInstance(CLSID_DragDropHelper, NULL, CLSCTX_INPROC_SERVER, IID_IDropTargetHelper,(LPVOID*)&m_DropHelper);

	m_hwnd = (HWND) hwnd;
	hr = RegisterDragDrop((HWND) m_hwnd, this);
	return S_OK;
}

STDMETHODIMP CBinder::Unbind()
{
  // ATL smart pointers automatically release on destruction
  RevokeDragDrop(m_hwnd);

	return S_OK;
}


HRESULT CBinder::DragEnter(IDataObject *pdto, DWORD grfKeyState, POINTL ptl, DWORD *pdwEffect)
{
	if (m_DropHelper) 
	{              
		/* Use the helper if we have one */
		POINT pt = { ptl.x, ptl.y };
		m_DropHelper->DragEnter(m_hwnd, pdto, &pt, *pdwEffect);
	} else
		*pdwEffect = DROPEFFECT_COPY;	
	
	/* Remember who is being dragged */
	m_DragObject = pdto;     

  return S_OK;
}

HRESULT CBinder::DragLeave()
{
  if (m_DropHelper) 
	{              
		/* Use the helper if we have one */
    m_DropHelper->DragLeave();
  }
	/* Nobody is being dragged any more */
	m_DragObject = NULL;

    
	return S_OK;
}

HRESULT CBinder::DragOver(DWORD grfKeyState, POINTL ptl, DWORD *pdwEffect)
{

  if (m_DropHelper) 
	{              
		/* Use the helper if we have one */
    POINT pt = { ptl.x, ptl.y };
    m_DropHelper->DragOver(&pt, *pdwEffect);
  } else
    *pdwEffect = DROPEFFECT_COPY;
  return S_OK;
}

HRESULT CBinder::Drop(IDataObject *pdto, DWORD grfKeyState, POINTL ptl, DWORD *pdwEffect)
{

	if (m_DropHelper) 
	{              
		/* Use the helper if we have one */
		POINT pt = { ptl.x, ptl.y };
		m_DropHelper->Drop(pdto, &pt, *pdwEffect);
	}
	/* Nobody is being dragged any more */
	m_DragObject = NULL;

	#define FORMATETC_MAX 256
	HRESULT hr;
	int iReturn;
	LPENUMFORMATETC lpEnumFmtEtc = NULL;
	FORMATETC rgfmtetc[FORMATETC_MAX];
  ULONG cFetched = 0, idxFetched;

  OLECHAR szCFName[MAX_PATH];
  OLECHAR szCFValue[MAX_PATH];
    
	// get enumerator
  hr = pdto->EnumFormatEtc(DATADIR_GET, (LPENUMFORMATETC FAR*)&lpEnumFmtEtc);
  if (FAILED(hr))
		return hr;

	//Enumerate the formats offered by the source.
  memset(rgfmtetc,0,sizeof(rgfmtetc[FORMATETC_MAX]) );
  hr = lpEnumFmtEtc->Next(FORMATETC_MAX, rgfmtetc, &cFetched); 
	bstrResult = L"";
  if ( SUCCEEDED(hr) || (cFetched > 0 && cFetched <= FORMATETC_MAX) )
  {
		// Loop over all formats offered by the source.
    for (idxFetched = 0; idxFetched < cFetched; idxFetched++)
    {
      iReturn = GetClipboardFormatNameW(rgfmtetc[idxFetched].cfFormat, szCFName,sizeof(szCFName));
			if (!iReturn)
				swprintf(szCFName, L"<unknown>");
			_ltow(rgfmtetc[idxFetched].cfFormat, szCFValue, 16);
			bstrResult += szCFValue;
			bstrResult += L"\t";
			bstrResult += szCFName;
			bstrResult += L"\n";
    }
	}

	if (lpEnumFmtEtc)
		lpEnumFmtEtc->Release();

	OpenClipboard(0);
	EmptyClipboard();
	CloseClipboard();

	hr = OleSetClipboard(pdto);
	hr = OleFlushClipboard();

		// fire event to indicate recieved results
	if (bstrResult.length()>0)
		Fire_OnDrop(bstrResult);

  *pdwEffect = DROPEFFECT_NONE;

  return S_OK;
}



STDMETHODIMP CBinder::get_Result(BSTR *pVal)
{
	*pVal = bstrResult;
	return S_OK;
}
